/// <summary>
///  Nivel: el nivel actual de juego
///  @author Nacho Cabanes
///  @see Menu Juego
/// </summary>

/* --------------------------------------------------         
   Parte de FreddyTwo
 
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  03-Feb-2012  Nacho Cabanes
                      Versin inicial: desglosado a partir de 
                        FreddyOne (Juego.cs)
   0.02  04-Feb-2012  Cristina Giner Olcina y Jorge Bernabeu Mira
                      Varios niveles consecutivos
   0.03  05-Mar-2012  Juan David Cabezuelo, Jorge Bernabeu Mira, Karol Campo Rangel
                      Convertir todos los Arrays en ArraysList
 ---------------------------------------------------- */

using System;  // Para Random
using System.IO;  // Para fichero de niveles
using System.Collections;//Para los Array List

public class Nivel
{
    // Atributos

    //private int tiempoMax;
    private int premiosRestantes;

    byte anchoFondo = 24;
    byte altoFondo = 17;
    short margenXFondo = 10;
    byte margenYFondo = 30;
    byte anchoCasillaFondo = 32;
    byte altoCasillaFondo = 32;

    ArrayList obstaculos;
    ArrayList enemigos;
    ArrayList premios;
    ArrayList fondos;
    int nivel;

    public char[, ,] fondo;


    public Nivel()
    {
        leerDesdeFichero();
        nivel = 0;
        RellenarArrayDeFondo();
    }

    private void leerDesdeFichero()
    {
        StreamReader ficheroMapas = File.OpenText("niveles.dat");
        string linea;

        linea = ficheroMapas.ReadLine();
        int numeroNiveles = Convert.ToInt32(linea);

        fondo = new char[numeroNiveles, altoFondo, anchoFondo];

        for (int nivelActual = 0; nivelActual < numeroNiveles; nivelActual++)
        {
            string separador = ficheroMapas.ReadLine();

            for (int lineaActual = 0; lineaActual < altoFondo; lineaActual++)
            {
                linea = ficheroMapas.ReadLine();
                for (int columnaActual = 0; columnaActual < anchoFondo; columnaActual++)
                    fondo[nivelActual, lineaActual, columnaActual] = linea[columnaActual];
                // Extraigo las columnas
            }
        }

        ficheroMapas.Close();
    }


    //funcion auxiliar
    public void RellenarArrayDeFondo()
    {
        // Y reservo espacio para el array que los contiene
        fondos = new ArrayList();
        obstaculos = new ArrayList();
        enemigos = new ArrayList();
        premios = new ArrayList();

        int posicFondo = 0, contEnemigos = 0, contObstaculos = 0, contPremios = 0;
        for (int fila = 0; fila < altoFondo; fila++)  // Fondo
            for (int col = 0; col < anchoFondo; col++)
                switch (fondo[nivel, fila, col])
                {
                    case '1':
                        fondos.Add(new ElemGrafico("pared.png", anchoCasillaFondo, altoCasillaFondo));
                        ((ElemGrafico)fondos[posicFondo]).MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        ((ElemGrafico)fondos[posicFondo]).SetVisible(true);
                        posicFondo++;
                        break;
                    case '2':
                        fondos.Add(new ElemGrafico("pared2.png", anchoCasillaFondo, altoCasillaFondo));
                        ((ElemGrafico)fondos[posicFondo]).MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        ((ElemGrafico)fondos[posicFondo]).SetVisible(true);
                        posicFondo++;
                        break;
                    case '3':
                        fondos.Add(new ElemGrafico("pared3.png", anchoCasillaFondo, altoCasillaFondo));
                        ((ElemGrafico)fondos[posicFondo]).MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        ((ElemGrafico)fondos[posicFondo]).SetVisible(true);
                        posicFondo++;
                        break;
                    case 'X':
                        premios.Add(new Premio());
                        ((ElemGrafico)premios[contPremios]).MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        ((ElemGrafico)premios[contPremios]).SetVisible(true);
                        contPremios++;
                        break;
                    case 'O':
                        obstaculos.Add(new Obstaculo());
                        ((ElemGrafico)obstaculos[contObstaculos]).MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        ((ElemGrafico)obstaculos[contObstaculos]).SetVisible(true);
                        contObstaculos++;
                        break;
                    case 'E':
                        enemigos.Add(new Enemigo());
                        ((ElemGrafico)enemigos[contEnemigos]).MoverA(margenXFondo + col * anchoCasillaFondo,
                            margenYFondo + fila * altoCasillaFondo);
                        ((ElemGrafico)enemigos[contEnemigos]).SetVisible(true);
                        contEnemigos++;
                        break;
                }
        premiosRestantes = premios.Count;
    }


    /// Deja todo como es necesario para una nueva partida
    public void Reiniciar()
    {
        nivel = 0;
        RellenarArrayDeFondo();
    }


    public void DibujarOculta()
    {
        for (int i = 0; i < fondos.Count; i++)  // Imgenes del fondo
            ((ElemGrafico)fondos[i]).DibujarOculta();

        for (int i = 0; i < obstaculos.Count; i++)  // Obstculos
            ((ElemGrafico)obstaculos[i]).DibujarOculta();

        for (int i = 0; i < enemigos.Count; i++)  // Enemigos
            ((ElemGrafico)enemigos[i]).DibujarOculta();

        for (int i = 0; i < premios.Count; i++)  // Premios
            ((ElemGrafico)premios[i]).DibujarOculta();

    }

    // Devuelve si es posible mover a ciertas coordenadas
    // (no hay obstculos)
    public bool EsPosibleMover(int x, int y, int xFin, int yFin)
    {
        // Veo si choca con algn elemento del fondo
        for (int i = 0; i < fondos.Count; i++)
            if (((ElemGrafico)fondos[i]).ColisionCon(x, y, xFin, yFin))
                return false;

        // Si no ha chocado con ninguno, es posible moverse a esa posicin
        return true;
    }

    public int ObtenerPuntosPosicion(int x, int y, int xFin, int yFin)
    {
        // Veo si recoge algn premio
        for (int i = 0; i < premios.Count; i++)
            if (((ElemGrafico)premios[i]).ColisionCon(x, y, xFin, yFin))
            {
                ((ElemGrafico)premios[i]).SetVisible(false);
                premiosRestantes--;
                if (premiosRestantes == 0)
                    Avanzar();
                return 10;
            }

        // O si toca algn obstculo
        for (int i = 0; i < obstaculos.Count; i++)
            if (((ElemGrafico)obstaculos[i]).ColisionCon(x, y, xFin, yFin))
                return -1;

        // O si choca con algn enemigo
        for (int i = 0; i < enemigos.Count; i++)
            if (((ElemGrafico)enemigos[i]).ColisionCon(x, y, xFin, yFin))
                return -1;

        // Si llego hasta aqui, es que no hay puntos que devolver
        return 0;

    }



    public int GetNumEnemigos()
    {
        return enemigos.Count;
    }


    public Enemigo GetEnemigo(int num)
    {
        return (Enemigo)enemigos[num];
    }


    public void Avanzar()
    {
        // Borro la pantalla y aviso del nuevo nivel
        Hardware.Pausa(200);
        Hardware.BorrarPantallaOculta();
        //Hardware.EscribirTextoOculta("Nivel: " + Convert.ToString(premiosVisibles + 1),
        //         350, 300, /* Coordenadas */ 255, 255, 255, /* Colores */ tipoDeLetra);
        Hardware.VisualizarOculta();
        Hardware.Pausa(1000);

        // Cambio los "ladrillos" del fondo
        if (nivel < 2)
            nivel++;
        else
            nivel = 0;

        RellenarArrayDeFondo();
    }

} /* Fin de la clase Nivel */

